@@ -34,6 +34,7 @@
 -export([sequence/4]).
 -export([loop/4]).
 -export([delete/3]).
+-export([delete/4]).
 -export([exception/2]).
 -export([passthrough/1]).
 -export([history/1]).
@@ -289,17 +290,33 @@ loop(Mod, Func, Ari, Loop) when is_list(Mod) ->
 %%
 %% Deletes the expectation for the function `Func' with the matching
 %% arity `Arity'.
+%% `Force' is a flag to delete the function even if it is passthrough.
+-spec delete(Mods, Func, Ari, Force) -> ok when
+      Mods :: Mod | [Mod],
+      Mod :: atom(),
+      Func :: atom(),
+      Ari :: byte(),
+      Force :: boolean().
+delete(Mod, Func, Ari, Force)
+  when is_atom(Mod), is_atom(Func), Ari >= 0 ->
+    meck_proc:delete_expect(Mod, Func, Ari, Force);
+delete(Mod, Func, Ari, Force) when is_list(Mod) ->
+    lists:foreach(fun(M) -> delete(M, Func, Ari, Force) end, Mod),
+    ok.
+
+%% @doc Deletes an expectation.
+%%
+%% Deletes the expectation for the function `Func' with the matching
+%% arity `Arity'.
+%% If the mock has passthrough enabled, this function restores the
+%% expectation to the original function. See {@link delete/4}.
 -spec delete(Mods, Func, Ari) -> ok when
       Mods :: Mod | [Mod],
       Mod :: atom(),
       Func :: atom(),
       Ari :: byte().
-delete(Mod, Func, Ari)
-  when is_atom(Mod), is_atom(Func), Ari >= 0 ->
-    meck_proc:delete_expect(Mod, Func, Ari);
-delete(Mod, Func, Ari) when is_list(Mod) ->
-    lists:foreach(fun(M) -> delete(M, Func, Ari) end, Mod),
-    ok.
+delete(Mod, Func, Ari) ->
+    delete(Mod, Func, Ari, false).
 
 %% @doc Throws an expected exception inside an expect fun.
 %%
@@ -332,6 +349,31 @@ passthrough(Args) when is_list(Args) ->
 %% arguments or non-existing function (undef), wrong arguments
 %% (function clause) or unexpected exceptions.
 %%
+%% Validation can detect:
+%%
+%% <ul>
+%%   <li>When a function was called with the wrong argument types
+%%       (`function_clause')</li>
+%%   <li>When an exception was thrown</li>
+%%   <li>When an exception was thrown and expected (via meck:exception/2),
+%%       which still results in `true' being returned</li>
+%% </ul>
+%%
+%% Validation cannot detect:
+%%
+%% <ul>
+%%   <li>When you didn't call a function</li>
+%%   <li>When you called a function with the wrong number of arguments
+%%       (`undef')</li>
+%%   <li>When you called an undefined function (`undef')</li>
+%% </ul>
+%%
+%% The reason Meck cannot detect these cases is because of how it is implemented.
+%% Meck replaces the module with a mock and a process that maintains the mock.
+%% Everything Meck get goes through that mock module. Meck does not insert
+%% itself at the caller level (i.e. in your module or in your test case), so it
+%% cannot know that you failed to call a module.
+%%
 %% Use the {@link history/1} or {@link history/2} function to analyze errors.
 -spec validate(Mods) -> boolean() when
       Mods :: Mod | [Mod],
@@ -601,7 +643,7 @@ is(MatcherImpl) ->
 %%
 %% If an occurrence of a function call irrespective of the calling process needs
 %% to be captured then `_' might be passed as `OptCallerPid', but it is better
-%% to use {@link capture/3} instead.
+%% to use {@link capture/5} instead.
 -spec capture(Occur, Mod, Func, OptArgsSpec, ArgNum, OptCallerPid) -> ArgValue when
       Occur :: first | last | pos_integer(),
       Mod :: atom(),
@@ -625,7 +667,7 @@ capture(Occur, Mod, Func, OptArgsSpec, ArgNum, OptCallerPid) ->
 %% retrieve the argument value passed when the function was called the first
 %% or the last time respectively.
 %%
-%% @equiv capture(Occur, '_', Mod, Func, OptArgsSpec, ArgNum)
+%% @equiv capture(Occur, Mod, Func, OptArgsSpec, ArgNum, '_')
 -spec capture(Occur, Mod, Func, OptArgsSpec, ArgNum) -> ArgValue when
       Occur :: first | last | pos_integer(),
       Mod::atom(),
